#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
@author: Stefan Hiemer
"""

import numpy as np

from integrals import eta_1,int_eta_dz,int_etaz_dz,intint_eta_dzdz,intint_etaz_dzdz

def get_delta(km, Th, Tm, rho_s, Us, h, Cs, T0):
    return (km * (Th - Tm)) / (rho_s * Us * (h + Cs * (Tm - T0)))

def Ft_newt_isoth(U0, eta0, delta, alpha, Ro, Ri):
    
    
    prefac = 2 * np.pi * (eta0 * U0) / delta
    term1 = 3 * ((1/np.cos(alpha)) * (delta**(-1)) * \
            ((2/3) * Ro**3 - Ri * Ro**2 + (1/3) * Ri**3)\
        + np.tan(alpha) * (Ro**2 /2 - 2*Ro*Ri + Ro**2 /2))
    
    term2 = - np.sin(alpha) * (Ro**2 - Ri**2) / 2
    
    return prefac*(term1 + term2)

def Fp_newt_isoth(U0, eta0, delta, alpha, Ro, Ri, etac, Lc,
                  debug):
    
    # 
    inlet_p = 8*np.pi*etac*U0*Lc*(Ro / Ri)**4
    
    Fp=3*np.pi*(U0*eta0)/(delta**2 *np.cos(alpha)**3) * \
       (((2*np.log(Ro/Ri) - 3/2) * Ro**4 + 2*Ri**2 *Ro**2 + Ri**4 /2)/delta \
        + np.sin(alpha) * ((2*np.log(Ro/Ri) - 7/3)*Ro**3 + 2*Ri*Ro**2 +\
                 Ri**2*Ro - 2/3 *Ri**3)) + inlet_p
    if debug:
        print("Pi",inlet_p)
    
    return Fp

def F_newt_isoth(km, Th, Tm, rho_s, rho_m, h, Cs, T0,
                 Us, eta0, alpha, Ro, Ri, etac, Lc,
                 debug=False):
    """
    

    Parameters
    ----------
    km : float
        melt heat conductivity.
    Th : float
        heater temperature in Kelvin.
    Tm : float
        melting temperature in K.
    rho_s : float
        density solid
    rho_m : float
        density melt.
    h : float
        heat of fusion.
    Cs : float
        heat capacity solid.
    T0 : float
        room temperature.
    Us : float
        feeder velocity.
    eta0 : float
        Newtonian viscosity.
    alpha : float
        angle.
    Ro : float
        outer radius.
    Ri : float
        inner radius.
    etac : float
        viscosity in capillary.
    Lc : float
        capillary length.

    Returns
    -------
    F: float
        feeder force.

    """
    
    if debug:
        print("Isothermal")
        
    # convert alpha from angle to radian
    alpha = alpha * np.pi/180
    
    #    
    U0= rho_s/rho_m * Us
    delta = get_delta(km, Th, Tm, rho_s, Us, h, Cs, T0)
    
    if debug:
        print("alpha",alpha,"\n"
              "U0",U0,
              "delta",delta)
    
    Ft = Ft_newt_isoth(U0, eta0, delta, alpha, Ro, Ri)
    Fp = Fp_newt_isoth(U0, eta0, delta, alpha, Ro, Ri, etac, Lc, debug)
    if debug:
        print("Ft",Ft) 
        print("Fp",Fp)
        
    return np.cos(alpha) * Fp - np.sin(alpha) * Ft

def eta(z,eta0,A,B):
    return eta0 * np.exp(A/(B+z))

def dhdz(z,U0,alpha,A,B,delta):
    return eta_1(z,A,B)*(z - int_etaz_dz(delta, A, B)/int_eta_dz(delta, A, B))

def dwdz(z,U0,alpha,A,B,delta):
    return (-1) * U0 * np.sin(alpha) * eta_1(z,A,B)/int_eta_dz(delta, A, B)

def h(z,A,B,delta):
    return ( int_etaz_dz(z, A, B) - int_eta_dz(z, A, B)/int_eta_dz(delta, A, B) * int_etaz_dz(delta, A, B))

def w(z,U0,alpha,A,B,delta):
    return (-1) * U0 * np.sin(alpha) * int_eta_dz(z, A, B)/int_eta_dz(delta, A, B)

def H(z,A,B,delta):
    return ( intint_etaz_dzdz(z, A, B) - intint_eta_dzdz(z, A, B) /int_eta_dz(delta, A, B) * int_etaz_dz(delta, A, B))

def W(z,U0,alpha,A,B,delta):
    return (-1) * U0 * np.sin(alpha) * intint_eta_dzdz(z, A, B) /int_eta_dz(delta, A, B)

def Ft_newt_noniso(U0, eta0, delta, alpha, Ro, Ri, A, B):
    
    
    Ft = 2*np.pi*eta0*eta(delta,eta0,A,B)*(-np.cos(alpha)**(-1) *(U0*((2/3)*Ro**3 \
        -Ri*Ro**2 +Ri**3 /3)-W(delta,U0,alpha,A,B,delta)*\
        (Ro**2 /2 -2*Ro*Ri+Ro**2 /2)) *\
        dhdz(delta,U0,alpha,A,B,delta)/H(delta,A,B,delta) \
        +dwdz(delta, U0, alpha, A, B, delta) *(Ro**2 - Ri**2)/2)
    
    return Ft

def Fp_newt_noniso(U0, eta0, delta, alpha, Ro, Ri, etac, Lc, A ,B,
                   debug):
    
    # inlet pressure
    inlet_p = 8*np.pi*etac*U0*Lc*(Ro ** 4)/(Ri**4)
    
    Fp =(-np.pi/4)*eta0*H(delta,A,B,delta)**(-1) * np.cos(alpha)**(-1) \
        *(U0*(Ro**4 * (2*np.log(Ro/Ri) - 3/2) + 2*Ri**2 *Ro**2 - Ri**4 /2) \
        - 2*W(delta,U0,alpha,A,B,delta)*((2*np.log(Ro/Ri) - 7/3)*Ro**3 + \
        2*Ri*Ro**2 + Ri**2 *Ro - 2/3 *Ri**3)) + inlet_p
            
    if debug:
        print("Pi",inlet_p)
        
    return Fp

def F_newt_vft(km, Th, Tm, rho_s, rho_m, h, Cs, T0,
               Us, eta0, eps, Tv, alpha, Ro, Ri, etac, Lc,
               debug=False):
    """
    Force for the Vogel-Fulcher-Tamann fluid. eta = eta0 exp(eps/(T-Tv))

    Parameters
    ----------
    km : float
        melt heat conductivity.
    Th : float
        heater temperature in Kelvin.
    Tm : float
        melting temperature in K.
    rho_s : float
        density solid
    rho_m : float
        density melt.
    h : float
        heat of fusion.
    Cs : float
        heat capacity solid.
    T0 : float
        room temperature.
    Us : float
        feeder velocity.
    eta0 : float
        Newtonian viscosity.
    eps : float
        Other parameter for VFT.
    Tv : float
        Vogel Fulcher Tamann temperature.
    alpha : float
        angle.
    Ro : float
        outer radius.
    Ri : float
        inner radius.
    etac : float
        viscosity in capillary.
    Lc : float
        capillary length.

    Returns
    -------
    F: float
        feeder force.

    """
    
    if debug:
        print("Vogel Fulcher")
        
    # convert alpha from angle to radian
    alpha = alpha * np.pi/180
    
    U0= rho_s/rho_m * Us
    delta = get_delta(km, Th, Tm, rho_s, Us, h, Cs, T0)
    
    if debug:
        print("alpha",alpha,
              "U0",U0,
              "delta",delta)
    
    A = eps/(Tm-Th) * delta
    B = (Th-Tv)/(Tm-Th) * delta
    if debug:
        print("A",A,
              "B",B)
    Ft = Ft_newt_noniso(U0, eta0, delta, alpha, Ro, Ri, A, B)
    
    Fp = Fp_newt_noniso(U0, eta0, delta, alpha, Ro, Ri, etac, Lc, A, B, debug)
    
    if debug:
        print("Ft",Ft,"\n",
              "Fp",Fp)
        
    return np.cos(alpha) * Fp - np.sin(alpha) * Ft

def F_newt_eyr(km, Th, Tm, rho_s, rho_m, h, Cs, T0,
               Us, eta0, Q, R, alpha, Ro, Ri, etac, Lc,
               debug=False):
    """
    Force for the Eyring Andrade fluid. eta = eta0 exp(Q/(RT))

    Parameters
    ----------
    km : float
        melt heat conductivity.
    Th : float
        heater temperature in Kelvin.
    Tm : float
        melting temperature in K.
    rho_s : float
        density solid
    rho_m : float
        density melt.
    h : float
        heat of fusion.
    Cs : float
        heat capacity solid.
    T0 : float
        room temperature.
    Us : float
        feeder velocity.
    eta0 : float
        Newtonian viscosity.
    Q : float
        energy barrier.
    R : float
        universal gas constant or Boltzmann constant.
    alpha : float
        angle.
    Ro : float
        outer radius.
    Ri : float
        inner radius.
    etac : float
        viscosity in capillary.
    Lc : float
        capillary length.

    Returns
    -------
    F: float
        feeder force.

    """
    
    if debug:
        print("Eyring")
        
    # convert alpha from angle to radian
    alpha = alpha * np.pi/180
    
    U0= rho_s/rho_m * Us
    delta = get_delta(km, Th, Tm, rho_s, Us, h, Cs, T0)
    if debug:
        print("alpha",alpha,"\n",
              "U0",U0,"\n",
              "delta",delta)
    
    A = Q/(R*(Tm-Th)) * delta
    B = Th/(Tm-Th) * delta
    if debug:
        print("A",A,"\n",
              "B",B)
    
    Ft = Ft_newt_noniso(U0, eta0, delta, alpha, Ro, Ri, A, B)
    
    Fp = Fp_newt_noniso(U0, eta0, delta, alpha, Ro, Ri, etac, Lc, A, B, debug)
    if debug:
        print("Ft",Ft,"\n",
              "Fp",Fp)
        
    return np.cos(alpha) * Fp - np.sin(alpha) * Ft

if __name__=="__main__":
    pass
